function [ awi_float ] = float( p_optimare_float )
%FLOAT Converts the given float in optimare structure to the structre of AWI.
%   This is only a temporary function and will be deprecated as soon as the
%   decoding process has been rewritten. During the decoding, optimare created a
%   variable structre that differs from the one AWI is using. This function
%   transfers all variables into the AWI format. In addition to that, it also
%   checks for incomplete profiles and always saves the profile with the biggest
%   size of CTD data.
%
%   Parameters:
%   p_optimare_float:   Structure that was created during the decoding process.
%
%   Returns:
%   awi_float:   Structure with the structure that is being used by AWI. False
%                if there is no data, no profiles or if something went wrong
%                during conversion.

%% Initialize return variables
awi_float = {};

%% Convert information to AWI structure

if ~isfield(p_optimare_float, 'DECODED_SBD')
    % no data available
    awi_float = false;
    return
end
% get fieldnames of every decoded profile
profile_fieldnames = fieldnames(p_optimare_float.DECODED_SBD);

% sort iridium messages by profile number
[profile_numbers_sorted, sorted_indices] = sort_iridium_messages(profile_fieldnames);

if ~sorted_indices
    % something went wrong, for example only incomplete iridium messages available
    awi_float = false;
    return
end

% get profiles with most CTD data if multiple ones have been transmitted
best_ctd_indices = filter_multiple_profiles(sorted_indices);

% count valid profiles
valid_profile_count = length(best_ctd_indices);


%% Get format data that is always the same
deployment_info_struct = deployment_info();
rafos_values_format_struct = rafos_values_format();
profile_header_struct = profile_header();
quality_control_header_struct = qualtiy_control_header();
profile_data_header_struct = profile_data_header();
surface_gps_data_format_struct = surface_gps_data_format();
iridium_positions_format_struct = iridium_positions_format();
iridium_data_format_struct = iridium_data_format();

% create variable, counting available valid profiles
valid_profiles = 0;

for o_profile = 1:valid_profile_count
    % get index of current profile, may not be same as o_profile because optimare
	% structure is not sorted by profile numbers
    current_index = best_ctd_indices(o_profile);
	
    if ~strcmp(p_optimare_float.DECODED_SBD.(profile_fieldnames{current_index}).DATATYPE, ...
            'Profile Message')
			% everything that is not a profile message is being ignored
            continue
    else
		% increase number of valid profiles
        valid_profiles = valid_profiles + 1;
    end
    try % converting the input structure
	
        %% FLOAT_IDENTIFICATION
        awi_float{1, valid_profiles}.FLOAT_IDENTIFICATION = ...
            float_identification(profile_fieldnames{current_index});
        %% OVERALL_MISSION_INFORMATION
        awi_float{1, valid_profiles}.OVERALL_MISSION_INFORMATION = ...
            overall_mission_information(profile_fieldnames{current_index});
        %% DEPLOYMENT_INFO
        awi_float{1, valid_profiles}.DEPLOYMENT_INFO = ...
            deployment_info_struct;
        %% PROFILE_TECHNICAL_DATA
        awi_float{1, valid_profiles}.PROFILE_TECHNICAL_DATA = ...
            profile_technical_data(profile_fieldnames{current_index});
        %% BOTTOM_VALUES_DURING_DRIFT
        awi_float{1, valid_profiles}.BOTTOM_VALUES_DURING_DRIFT = ...
            bottom_values_during_drift(profile_fieldnames{current_index});
        %% RAFOS_VALUES_FORMAT
        awi_float{1, valid_profiles}.RAFOS_VALUES_FORMAT = ...
            rafos_values_format_struct;
        %% RAFOS_VALUES
        awi_float{1, valid_profiles}.RAFOS_VALUES = ...
            rafos_values(profile_fieldnames{current_index});
        %% PROFILE_HEADER
        awi_float{1, valid_profiles}.PROFILE_HEADER = ...
            profile_header_struct;
        %% QUALITY_CONTROL_HEADER
        awi_float{1, valid_profiles}.QUALITY_CONTROL_HEADER = ...
            quality_control_header_struct;
        %% PROFILE_DATA_HEADER
        awi_float{1, valid_profiles}.PROFILE_DATA_HEADER = ...
            profile_data_header_struct;
        %% PROFILE_DATA
        awi_float{1, valid_profiles}.PROFILE_DATA = ...
            profile_data(profile_fieldnames{current_index});
			
        % if there is no profile data, clean up and go to next profile
        if isempty(awi_float{1, valid_profiles}.PROFILE_DATA)
            % skip profiles without data
            awi_float{1, valid_profiles} = [];
            valid_profiles = valid_profiles - 1;
            continue
        end
		
        %% SURFACE_GPS_DATA_FORMAT
        awi_float{1, valid_profiles}.SURFACE_GPS_DATA_FORMAT = ...
            surface_gps_data_format_struct;
        %% SURFACE_GPS_DATA
        awi_float{1, valid_profiles}.SURFACE_GPS_DATA = ...
            surface_gps_data(profile_fieldnames{current_index});
        %% IRIDIUM_POSITIONS_FORMAT
        awi_float{1, valid_profiles}.IRIDIUM_POSITIONS_FORMAT = ...
            iridium_positions_format_struct;
        %% IRIDIUM_POSITIONS & IRIDIUM_DATA
        [ iridium_positions, iridium_best ] = ...
            iridium_positions_and_data(profile_fieldnames{current_index});
        awi_float{1, valid_profiles}.IRIDIUM_POSITIONS = iridium_positions;
        clear iridium_positions
        %% IRIDIUM_DATA_FORMAT
        awi_float{1, valid_profiles}.IRIDIUM_DATA_FORMAT = ...
            iridium_data_format_struct;
        awi_float{1, valid_profiles}.IRIDIUM_DATA = iridium_best;
        clear iridium_best
        
        
    catch exception
        % something went wrong, so complain by returning false
        awi_float = false;
        return;
    end
    
    
end

% if there is no profile data at all, return false
if valid_profiles == 0
    awi_float = false;
    return;
end

    function [ value ] = store_if_available(p_search, p_needle)
        % if the given needle is a member of search, return the containing value.
        if isfield(p_search, p_needle)
            value = p_search.(p_needle);
            switch class(value)
                case 'char'
                    value = strtrim(value);
            end
            return
        else
            value = NaN;
            return
        end
    end


%% FLOAT_IDENTIFICATION

    function [ awi_struct ] = float_identification(p_profile_fieldname)
        
        % abbreveations usually not allowed, this time used for better overview
        awi_struct = {};
        
        awi_struct.internal_ID_number = store_if_available( ...
            p_optimare_float.DECODED_SBD, 'internal_ID_number');
        
        if isfield( ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname),'DEVICE') ...
                && strcmp(strtrim( ...
                  p_optimare_float.DECODED_SBD.(p_profile_fieldname).DEVICE),'NEMO Standard' ...
                  )
            awi_struct.transmission_type = 'IRIDIUM SBD';
        else
            awi_struct.transmission_type = NaN;
        end
        
        if isfield(p_optimare_float.GENERAL_INFO, 'IMEI')
            awi_struct.transmission_id_number_dec = ...
                p_optimare_float.GENERAL_INFO.IMEI;
            awi_struct.transmission_id_number_hex = NaN;
        else
            awi_struct.transmission_id_number_dec = NaN;
            awi_struct.transmission_id_number_hex = NaN;
        end
        
        awi_struct.wmo_id_number = store_if_available( ...
            p_optimare_float.GENERAL_INFO, 'WMO_ID');
        
        awi_struct.wmo_instrument_type = store_if_available( ...
            p_optimare_float.GENERAL_INFO, 'WMO_INSTRUMENT_TYPE');
        
        
        
        awi_struct.wmo_recorder_type = store_if_available( ...
            p_optimare_float.GENERAL_INFO, 'WMO_RECORD_TYPE');
        
        awi_struct.instrument_type = store_if_available( ...
            p_optimare_float.GENERAL_INFO, 'INSTRUMENT_TYPE');
        
        awi_struct.float_manufacturer = store_if_available( ...
            p_optimare_float.GENERAL_INFO, 'FLOAT_MANUFACTURER');
        
        awi_struct.float_serial_number = store_if_available( ...
            p_optimare_float.GENERAL_INFO, 'SERIAL');
        
        awi_struct.ice_detection_software = store_if_available( ...
            p_optimare_float.SENSOR_INFO, 'ICE_DETECTION');
        
        
        awi_struct.float_provider = store_if_available( ...
            p_optimare_float.PROVIDER_DEPLOYER_INFO, 'PROVIDER_NAME');
        
        
        awi_struct.float_provider_institution = store_if_available( ...
            p_optimare_float.PROVIDER_DEPLOYER_INFO, 'PROVIDER_INSTITUTE');
        
        awi_struct.originating_country = store_if_available( ...
            p_optimare_float.PROVIDER_DEPLOYER_INFO, 'PROVIDER_COUNTRY');
        
        
    end

%% OVERALL_MISSION_INFORMATION

    function [ awi_struct ] = overall_mission_information(p_profile_fieldname)
        
        awi_struct = {};
        
        if isfield(p_optimare_float.DECODED_SBD.(p_profile_fieldname), 'TIME_CYCLE')
            awi_struct.total_cycle = [ ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname).TIME_CYCLE/(60*24) ...
                0 0];
        else
            awi_struct.total_cycle = NaN;
        end
        
        awi_struct.down_time = NaN;
        awi_struct.up_time = NaN;
        awi_struct.transmission_repetition_rate = NaN;
        awi_struct.clock_drift = NaN;
        awi_struct.nominal_drift_depth = NaN;
        awi_struct.max_departure_from_drift_depth = NaN;
        
        awi_struct.nominal_profile_depth = store_if_available( ...
            p_optimare_float.MISSION_PARAMETERS, 'PROFILE_PRESSURE');
        
        if isfield(p_optimare_float.DEPLOYMENT_INFO, 'START_DATE')
            tmp_mat = strsplit(p_optimare_float.DEPLOYMENT_INFO.START_DATE, '.');
            awi_struct.start_date = [ ...
                str2double(tmp_mat{1}) ...
                str2double(tmp_mat{2}) ...
                str2double(tmp_mat{3})];
            clear tmp_mat
        else
            awi_struct.start_date = NaN;
        end
        
        if isfield(p_optimare_float.DEPLOYMENT_INFO, 'START_TIME')
            tmp_mat = strsplit(p_optimare_float.DEPLOYMENT_INFO.START_TIME, ':');
            awi_struct.start_time = [ ...
                str2double(tmp_mat{1}) ...
                str2double(tmp_mat{2})];
            clear tmp_mat
        else
            awi_struct.start_time = NaN;
        end
        
        awi_struct.RAFOS_clock_offset_at_start = NaN;
        awi_struct.start_surface_pressure = NaN;
        
        awi_struct.ice_detection_temperature_threshold = store_if_available( ...
            p_optimare_float.MISSION_PARAMETERS, 'ICE_TEMPERATURE');
        
        
    end

%% DEPLOYMENT_INFO
    function [ awi_struct ] = deployment_info()
        
        awi_struct.float_deployer = store_if_available( ...
            p_optimare_float.PROVIDER_DEPLOYER_INFO, 'DEPLOYER_NAME');
        
        awi_struct.float_deployer_institution = store_if_available( ...
            p_optimare_float.PROVIDER_DEPLOYER_INFO, 'DEPLOYER_INSTITUTE');
        
        awi_struct.deployment_platform_type = store_if_available( ...
            p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_PLATFORM');
        
        awi_struct.deployment_platform_code = store_if_available( ...
            p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_PLATFORM_SIGN');
        
        if isfield(p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_PLATFORM_EXPEDITION')
            tmp_string = strsplit( ...
                p_optimare_float.DEPLOYMENT_INFO.DEPLOYMENT_PLATFORM_EXPEDITION, ...
                '/');
            if length(tmp_string) == 2
                awi_struct.deployment_platform_cruise_name = tmp_string{1};
                awi_struct.deployment_platform_leg = tmp_string{2};
            else
                awi_struct.deployment_platform_cruise_name = ...
                    p_optimare_float.DEPLOYMENT_INFO.DEPLOYMENT_PLATFORM_EXPEDITION;
                awi_struct.deployment_platform_leg = 'none';
            end
            clear tmp_string
        else
            awi_struct.deployment_platform_cruise_name = NaN;
            awi_struct.deployment_platform_leg = 'none';
        end
        
        awi_struct.deployment_ctd_station = NaN;
        
        if isfield(p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_DATE')
            tmp_string = strsplit( ...
                p_optimare_float.DEPLOYMENT_INFO.DEPLOYMENT_DATE, ...
                '.');
            awi_struct.deployment_date = [ str2double(tmp_string{1}) ...
                str2double(tmp_string{2}) ...
                str2double(tmp_string{3}) ];
            clear tmp_string
        else
            awi_struct.deployment_date = NaN;
        end
        
        if isfield(p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_TIME')
            tmp_string = strsplit( ...
                p_optimare_float.DEPLOYMENT_INFO.DEPLOYMENT_TIME, ...
                ':');
            awi_struct.deployment_time = [ str2double(tmp_string{1}) ...
                str2double(tmp_string{2}) ];
            clear tmp_string
        else
            awi_struct.deployment_time = NaN;
        end

        if isfield(p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_LAT') ...
                && isfield(p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_LON')
            awi_struct.deployment_position = [ ...
                p_optimare_float.DEPLOYMENT_INFO.DEPLOYMENT_LAT ...
                p_optimare_float.DEPLOYMENT_INFO.DEPLOYMENT_LON];
        else
            awi_struct.deployment_position = NaN;
        end

        awi_struct.deployment_speed = NaN;
        
        awi_struct.deployment_sea_state = store_if_available( ...
            p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_SEA_STATE');
        
        awi_struct.deployment_wave_height = NaN;
        
        awi_struct.deployment_ice_coverage = store_if_available( ...
            p_optimare_float.DEPLOYMENT_INFO, 'DEPLOYMENT_SEA_ICE');
        
        awi_struct.delay_of_first_down_time = NaN;
        
    end

%% PROFILE_TECHNICAL_DATA

    function [ awi_struct ] = profile_technical_data(p_profile_fieldname)
        
        awi_struct = {};
        
        awi_struct.xmit_profile_number = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PROFILE_NUMBER');
        
        awi_struct.xmit_serial_number = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'SERIAL');
        
        awi_struct.xmit_upcast_status = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'STATUS_UPCAST');
        
        awi_struct.xmit_older_profiles_not_send = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PROFILE_OLDER');
        
        awi_struct.xmit_motor_errors_count = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ERROR_MOTOR');
        
        awi_struct.xmit_sbe42_ctd_errors = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ERROR_CTD');
        
        awi_struct.xmit_o2_optode_errors = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ERROR_OPTODE');
        
        awi_struct.xmit_rafos_errors = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ERROR_RAFOS');
        
        awi_struct.xmit_cpu_battery_voltage = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'BATT_CPU');
        
        awi_struct.xmit_pump_battery_voltage = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'BATT_PMP');
        
        awi_struct.xmit_motor_current = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'CURR_MOTOR');
        
        awi_struct.xmit_motor_current_mean = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'CURR_MOTOR_MEAN');
        
        awi_struct.xmit_internal_pressure_surface = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PRES_TUBE_SURFACE');
        
        awi_struct.xmit_internal_pressure_depth = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PRES_TUBE_DEPTH');
        
        awi_struct.xmit_pressure_offset = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PRES_OFFSET');
        
        awi_struct.xmit_surface_pressure = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PRES_SURFACE');
        
        awi_struct.xmit_parking_pressure_median = median(store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'DS2_PRES'));
        
        awi_struct.xmit_depth_pressure = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PRES_DEPTH');
        
        awi_struct.xmit_depth_pressure_max = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PRES_MAX');
        
        awi_struct.xmit_profile_recovery = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'RECOVERY_START');
        
        awi_struct.xmit_piston_counts_surface = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PIST_SURFACE');
        
        awi_struct.xmit_piston_counts_parking = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PIST_PARKING');
        
        awi_struct.xmit_piston_counts_calc = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PIST_CALC');
        
        awi_struct.xmit_piston_counts_eop = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'PIST_EOP');
        
        awi_struct.xmit_descent_start_time = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'MCNT_DESCENT');
        
        awi_struct.xmit_parking_start_time = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'MCNT_PARKING');
        
        awi_struct.xmit_upcast_start_time = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'MCNT_SAG');
        
        awi_struct.xmit_ascent_start_time = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'MCNT_ASCENT');
        
        awi_struct.xmit_ascent_end_time = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'MCNT_ASCENT_END');
        
        awi_struct.xmit_surface_start_time = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'MCNT_SURFACE');
        
        if isfield(p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
                'DATA_RECORDS2') ...
                && length(p_optimare_float.DECODED_SBD.(p_profile_fieldname).DATA_RECORDS2) == 5
            awi_struct.xmit_profile_length_1 = ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname).DATA_RECORDS2(1);
            awi_struct.xmit_profile_length_2 = ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname).DATA_RECORDS2(2);
            awi_struct.xmit_profile_length_3 = ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname).DATA_RECORDS2(3);
            awi_struct.xmit_profile_length_4 = ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname).DATA_RECORDS2(4);
            awi_struct.xmit_profile_length_5 = ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldname).DATA_RECORDS2(5);
        else
            awi_struct.xmit_profile_length_1 = NaN;
            awi_struct.xmit_profile_length_2 = NaN;
            awi_struct.xmit_profile_length_3 = NaN;
            awi_struct.xmit_profile_length_4 = NaN;
            awi_struct.xmit_profile_length_5 = NaN;
        end
        
        awi_struct.xmit_surface_detection_GPS = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ICEGPS_TIME');
        
        awi_struct.xmit_surface_detection_Iridium = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ICEIRDM_TIME');
        
        awi_struct.xmit_airpump_runtime = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'AIRPUMP_RUNTIME');
        
        awi_struct.xmit_ice_detection_temp_median = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), ...
            'ICE_TEMPMEDIAN');
        
    end

%% BOTTOM_VALUES_DURING_DRIFT
    function [ awi_struct ] = bottom_values_during_drift(p_profile_fieldname)
        
        awi_struct = {};
        
        awi_struct.xmit_bottom_pressure = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), 'DS2_PRES');
        
        awi_struct.xmit_bottom_salt = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), 'DS2_SAL');
        
        awi_struct.xmit_bottom_temperature = store_if_available( ...
            p_optimare_float.DECODED_SBD.(p_profile_fieldname), 'DS2_TEMP');
        
    end

%% RAFOS_VALUES_FORMAT

    function [ awi_struct ] = rafos_values_format()
        
        awi_struct = { ...
            'status';  'rtc_year '; 'rtc_month'; 'rtc_day'; 'rtc_hour'; ...
            'rtc_minute'; 'rtc_second'; '1STCORHEIGHT'; '1STTRAVELTIME'; ...
            '2NDCORHEIGHT'; '2NDTRAVELTIME'; '3RDCORHEIGHT'; '3RDTRAVELTIME'; ...
            '4THCORHEIGHT'; '4THTRAVELTIME'; '5THCORHEIGHT'; '5THTRAVELTIME'; ...
            '6THCORHEIGHT'; '6THTRAVELTIME'; 'pressure'; 'salinity'; 'temp' ...
            }';
        
    end

%% RAFOS_VALUES

    function [ awi_struct ] = rafos_values(p_profile_fieldname)
        
        p = p_optimare_float.DECODED_SBD.(p_profile_fieldname);
        if isfield(p, 'DS3_DATE_year') && isfield(p, 'DS3_DATE_month') ...
                && isfield(p, 'DS3_DATE_day') && isfield(p, 'DS3_TIME') ...
                && isfield(p, 'DS3_STATUS') && isfield(p, 'DS3_RAFOSAMP') ...
                && isfield(p, 'DS3_RAFOSRANK') && isfield(p, 'DS3_PRES') ...
                && isfield(p, 'DS3_SAL') && isfield(p, 'DS3_TEMP')
            
            y = (1/(4836-4464) .* p.DS3_DATE_year) + 2000;
            mon = p.DS3_DATE_month;
            d = p.DS3_DATE_day;
            
            h = floor(p.DS3_TIME./3600);
            m = floor((p.DS3_TIME - (h*3600))/60);
            s = uint8(mod(p.DS3_TIME,60));
            
            prefix = [p.DS3_STATUS', y', mon', d', h', m', double(s')];
            
            awi_struct = [ ...
                prefix, ...
                p.DS3_RAFOSAMP(:, 1), p.DS3_RAFOSRANK(:, 1), ...
                p.DS3_RAFOSAMP(:, 2), p.DS3_RAFOSRANK(:, 2), ...
                p.DS3_RAFOSAMP(:, 3), p.DS3_RAFOSRANK(:, 3), ...
                p.DS3_RAFOSAMP(:, 4), p.DS3_RAFOSRANK(:, 4), ...
                p.DS3_RAFOSAMP(:, 5), p.DS3_RAFOSRANK(:, 5), ...
                p.DS3_RAFOSAMP(:, 6), p.DS3_RAFOSRANK(:, 6), ...
                p.DS3_PRES', p.DS3_SAL', p.DS3_TEMP' ...
                ];
        else
            awi_struct = [];
        end
        
        
    end

%% PROFILE_HEADER

    function [ awi_struct ] = profile_header()
        
        awi_struct = {};
        
        awi_struct.number_of_columns = 13;
        awi_struct.column_1 = 'line_number';
        awi_struct.column_2 = 'hour';
        awi_struct.column_3 = 'minute';
        awi_struct.column_4 = 'second';
        awi_struct.column_5 = 'temperature [deg C]';
        awi_struct.column_6 = 'temperature flag';
        awi_struct.column_7 = 'pressure [dbar]';
        awi_struct.column_8 = 'pressure flag';
        awi_struct.column_9 = 'salinity [psu]';
        awi_struct.column_10 = 'salinity flag';
        awi_struct.column_11 = 'temperature raw';
        awi_struct.column_12 = 'pressure raw';
        awi_struct.column_13 = 'salinity raw';
        awi_struct.temperature_conversion_equation = 'T=cnts/1000, if(cnts>62535),cnts=cnts-65536';
        awi_struct.pressure_conversion_equation = 'P=cnts/10';
        awi_struct.salinity_conversion_equation = 'S=cnts/1000';
        
    end

%% QUALITY_CONTROL_HEADER

    function [ awi_struct ] = qualtiy_control_header()
        
        awi_struct = {};
        
        awi_struct.overall_flag_for_T = 0;
        awi_struct.overall_flag_for_P = 0;
        awi_struct.overall_flag_for_S = 0;
        awi_struct.maximum_speed_check_flag = 0;
        awi_struct.maximum_analysis_check_T_flag = 0;
        awi_struct.maximum_climatology_T_flag = 0;
        awi_struct.maximum_gross_check_T_flag = 0;
        awi_struct.maximum_vertical_gradient_T_flag = 0;
        awi_struct.maximum_spike_check_T_flag = 0;
        awi_struct.maximum_analysis_check_S_flag = 0;
        awi_struct.maximum_climatology_S_flag = 0;
        awi_struct.maximum_gross_check_S_flag = 0;
        awi_struct.maximum_vertical_gradient_S_flag = 0;
        awi_struct.maximum_spike_check_S_flag = 0;
        awi_struct.sigma_used_for_T_climatology = 10;
        awi_struct.sigma_used_for_T_analysis = 10;
        awi_struct.sigma_used_for_S_climatology = 10;
        awi_struct.sigma_used_for_S_analysis = 10;
        
    end

%% PROFILE_DATA_HEADER

    function [ awi_struct ] = profile_data_header()
        
        awi_struct = { ...
            '#'; 'hour'; 'minute'; 'second'; 'temp'; 'temp_flag'; 'pressure'; ...
            'pres_flag'; 'salinity'; 'sal_flag'; 'temp_raw'; 'pres_raw'; ...
            'sal_raw'; 'light_442_nm'; 'light_550_nm'; 'light_676_nm' ...
            }';
        
    end

%% PROFILE_DATA

    function [ awi_struct ] = profile_data(p_profile_fieldname)
        
        p = p_optimare_float.DECODED_SBD.(p_profile_fieldname);
        if isfield(p, 'DS4_TIME') && isfield(p, 'DS4_TEMP') ...
                && isfield(p, 'DS4_PRES') && isfield(p, 'DS4_SAL')
                        
            h = floor(p.DS4_TIME./3600);
            m = floor((p.DS4_TIME - (h*3600))/60);
            s = uint8(mod(p.DS4_TIME, 60));
            profile_numbers = 1:length(h);
            flags = zeros(1, length(h));
            
            prefix = [profile_numbers', h', m', double(s')];
            
            % somtimes LICH1-3 has different length than TEMP, PRES & SAL
            if ~isfield(p, 'DS4_LICH1') || ~isfield(p, 'DS4_LICH2') ...
                 || ~isfield(p, 'DS4_LICH3') || (length(p.DS4_LICH1) ~= length(p.DS4_TEMP))
                LICH1 = flags;
                LICH2 = flags;
                LICH3 = flags;
            else
                LICH1 = p.DS4_LICH1;
                LICH2 = p.DS4_LICH2;
                LICH3 = p.DS4_LICH3;
            end
            
            awi_struct = [ ...
                prefix, ...
                p.DS4_TEMP', flags', ...
                p.DS4_PRES', flags', ...
                p.DS4_SAL', flags', ...
                p.DS4_TEMP', p.DS4_PRES', p.DS4_SAL', ...
                LICH1', LICH2', LICH3' ...
                ];
        else
            awi_struct = [];
        end
        
        
    end

%% SURFACE_GPS_DATA_FORMAT

    function [ awi_struct ] = surface_gps_data_format()
        
        awi_struct = { ...
            '#'; 'rtc_year'; 'rtc_month'; 'rtc_day'; 'rtc_hour'; 'rtc_minute'; ...
            'rtc_second'; 'GPS_year'; 'GPS_month'; 'GPS_day'; 'GPS_hour'; ...
            'GPS_minute'; 'GPS_second'; 'lat'; 'lon' ...
            }';
        
    end

%% SURFACE_GPS_DATA

    function [ awi_struct ] = surface_gps_data(p_profile_fieldname)
        
        p = p_optimare_float.DECODED_SBD.(p_profile_fieldname);
        if isfield(p, 'DS5_GPS') && isfield(p, 'DS5_RTC') ...
                && isfield(p, 'DS5_LAT') && isfield(p, 'DS5_LON')
            
            gps_datevec = datevec(p.DS5_GPS);
            % check if gps date is correct
            if gps_datevec(1) == 1999 && gps_datevec(2) == 12 ...
                    && gps_datevec(3) == 31
                gps_datevec = [NaN, NaN, NaN, NaN, NaN, NaN];
            end
            
            awi_struct = [ ...
                1, ...
                datevec(p.DS5_RTC), ...
                gps_datevec, ...
                p.DS5_LAT, ...
                p.DS5_LON ...
                ];
        else
            awi_struct = [];
        end
        
        
    end

%% IRIDIUM_POSITIONS_FORMAT

    function [ awi_struct ] = iridium_positions_format()
        
        awi_struct = { ...
            'sbd_lat'; 'sbd_lon'; 'sbd_cep'; ...
            'yyyy'; 'mm'; 'dd'; ...
            'hh'; 'mm'; 'ss'
            }';
        
    end

%% IRIDIUM_POSITIONS

    function [ iridium_positions, iridium_best ] = ...
            iridium_positions_and_data(p_profile_fieldname)
        
        p = p_optimare_float.DECODED_SBD.(p_profile_fieldname);
        sbd_info = p_optimare_float.SBD_MESSAGES_INFO;
        
        if ~isfield(p_optimare_float.TRANSMISSION_INFO, 'MOMSN') ...
                || ~isfield(p_optimare_float.TRANSMISSION_INFO, 'TIMEOFSESSION') ...
                || ~isfield(p_optimare_float.TRANSMISSION_INFO, 'IRIDIUM_LAT') ...
                || ~isfield(p_optimare_float.TRANSMISSION_INFO, 'IRIDIUM_LON') ...
                || ~isfield(p_optimare_float.TRANSMISSION_INFO, 'CEPRAD')
            return; % one of the fields is not available
        end
        
        % get transmission info
        momsn = p_optimare_float.TRANSMISSION_INFO.MOMSN;
        time_of_session = p_optimare_float.TRANSMISSION_INFO.TIMEOFSESSION;
        lat = p_optimare_float.TRANSMISSION_INFO.IRIDIUM_LAT;
        lon = p_optimare_float.TRANSMISSION_INFO.IRIDIUM_LON;
        cep = p_optimare_float.TRANSMISSION_INFO.CEPRAD;
        
        sbd_info_count = length(sbd_info);
    
        ir_lat = [];
        ir_lon = [];
        ir_cep = [];
        ir_time = [];
        ir_time_datevec = [];
        
        % get all profile numbers of the transmitted iridium messsages
        table_sbd_info = struct2table(sbd_info);
        profile_number_lsb = table_sbd_info(:, {'profilenumber_lsb'});
        % transform them to be able to use it as comparison
        profile_number_lsb = table2array(profile_number_lsb);
        % compare them to the current one
        tmp_logical_array = (profile_number_lsb(:) == p.PROFILE_NUMBER);
        % get all corresponding messages
        iridium_messages = table_sbd_info(tmp_logical_array, :);
        iridium_messages_count = size(iridium_messages, 1);
        
        for o_iridium = 1:iridium_messages_count
            tmp_logical_array = (momsn == iridium_messages(o_iridium, :).momsn);
            % get current lat
            current_lat = lat(tmp_logical_array);
            current_lon = lon(tmp_logical_array);
            current_cep = cep(tmp_logical_array);
            current_time = time_of_session(tmp_logical_array);
            current_time_datevec = datevec(current_time);
            if length(current_lat) > 1
                % get line with best cep
                [~, index] = min(current_cep);
                current_lat = nanmean(current_lat(index));
                current_lon = nanmean(current_lon(index));
                current_cep = nanmean(current_cep(index));
                current_time = nanmean(current_time(index));
                current_time_datevec = datevec(current_time);
            end
            ir_lat = [ir_lat; current_lat];
            ir_lon = [ir_lon; current_lon];
            ir_cep = [ir_cep; current_cep];
            ir_time = [ir_time; current_time];
            ir_time_datevec = [ir_time_datevec; current_time_datevec];
        end
        
        
        if isempty(ir_lat) || isempty(ir_lon) || isempty(ir_cep) ...
                || isempty(ir_time)
            iridium_positions = [];
            iridium_best = [];
            return
%         else
%             % delete all cep > 10, if there is nothing left store all to prevent
%             % NaN's
%             tmp_logical_array = (ir_cep < 10);
%             if sum(tmp_logical_array) > 0
%                 ir_lat = ir_lat(tmp_logical_array);
%                 ir_lon = ir_lon(tmp_logical_array);
%                 ir_cep = ir_cep(tmp_logical_array);
%                 ir_time = ir_time(tmp_logical_array);
%                 ir_time_datevec = datevec(ir_time);
%             end
        end
        
        % create return structure for all positions
        iridium_positions = [ ...
            ir_lat, ir_lon, ir_cep, ir_time_datevec ...
            ];
        
        % get best position
        best_ceps = unique(ir_cep);
        ix_best = (ir_cep == min(best_ceps));
        ir_lat_mean = nanmean(ir_lat(ix_best));
        ir_lon_mean = nanmean(ir_lon(ix_best));
        ir_cep_mean = nanmean(ir_cep(ix_best));
        %ir_time_mean = nanmean(ir_time(ix_best));
        
        iridium_best = [ ...
            ir_lat_mean, ir_lon_mean, ir_cep_mean, ir_time_datevec(end, :)
            ];
        
    end

%% IRIDIUM_DATA_FORMAT

    function [ awi_struct ] = iridium_data_format()
        
        awi_struct = { ...
            'mean_lat'; 'mean_lon'; 'mean_cep'; ...
            'yyyy'; 'mm'; 'dd'; ...
            'hh'; 'mm'; 'ss'
            }';
        
    end

%% SORT IRIDIUM MESSAGES

    function [ profile_numbers_sorted, indices ] = sort_iridium_messages(p_profile_fieldnames)
        profile_numbers_unsorted = [];
        tmp_valid_profile_count = 0;
        % collect all profile numbers
        for o_fields = 1:length(p_profile_fieldnames)
            current_profile = ...
                p_optimare_float.DECODED_SBD.(p_profile_fieldnames{o_fields});
            if ~strcmp(p_optimare_float.DECODED_SBD.(p_profile_fieldnames{o_fields}).DATATYPE, 'Profile Message') ...
                    || ~isfield(p_optimare_float.DECODED_SBD.(p_profile_fieldnames{o_fields}), 'PROFILE_NUMBER') ...
                    || isnan(p_optimare_float.DECODED_SBD.(p_profile_fieldnames{o_fields}).PROFILE_NUMBER)
                continue;
            end
            tmp_valid_profile_count = tmp_valid_profile_count + 1;
            profile_numbers_unsorted(tmp_valid_profile_count, :) = ...
                [p_optimare_float.DECODED_SBD.(p_profile_fieldnames{o_fields}).PROFILE_NUMBER, ...
                o_fields];
        end
        % sort them
        if ~isempty(profile_numbers_unsorted)
            [profile_numbers_sorted, indices] = sortrows(profile_numbers_unsorted, 1);
            indices = profile_numbers_sorted(:, 2);
        else
            indices = false;
            profile_numbers_sorted = false;
        end
    end

    function [best_ctd_indices] = filter_multiple_profiles(sorted_indices)
        % initialize return variable
        best_ctd_indices = [];
        best_indices_count = 0;
        % get profile number of the first element
        if isfield(p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(1)}), 'DATA_RECORDS2') ...
                && p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(1)}).DATA_RECORDS2(4) > 0
            % if it is < 0, there is no possibility to compare CTD data
            best_indices_count = best_indices_count + 1;
            last_profile_number = p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(1)}).PROFILE_NUMBER;
            best_ctd_indices(best_indices_count) = sorted_indices(1);
        end
        
        for o_index = 2:length(sorted_indices)
            if ~isfield(p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}), 'DATA_RECORDS2') ...
                || p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DATA_RECORDS2(4) == 0
                continue
            end
            % get current profile number
            current_profile_number = p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).PROFILE_NUMBER;
            if best_indices_count > 0 && current_profile_number == last_profile_number
                % check which ctd data is better
                current_ctd_length = p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DATA_RECORDS2(4);
                last_ctd_length = p_optimare_float.DECODED_SBD.(profile_fieldnames{best_ctd_indices(best_indices_count)}).DATA_RECORDS2(4);
                if current_ctd_length > last_ctd_length
                    best_ctd_indices(best_indices_count) = sorted_indices(o_index);
                elseif current_ctd_length == last_ctd_length
                    % take the one without or less NaN's
                    tmp_current = [ ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DS4_TIME; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DS4_FORMAT; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DS4_PRES; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DS4_TEMP; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{sorted_indices(o_index)}).DS4_SAL ...
                    ];
                    tmp_last = [ ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{best_ctd_indices(best_indices_count)}).DS4_TIME; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{best_ctd_indices(best_indices_count)}).DS4_FORMAT; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{best_ctd_indices(best_indices_count)}).DS4_PRES; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{best_ctd_indices(best_indices_count)}).DS4_TEMP; ...
                        p_optimare_float.DECODED_SBD.(profile_fieldnames{best_ctd_indices(best_indices_count)}).DS4_SAL ...
                        ];
                    sum_current = sum(sum(isnan(tmp_current)));
                    sum_last = sum(sum(isnan(tmp_last)));
                    if sum_last > sum_current
                        best_ctd_indices(best_indices_count) = sorted_indices(o_index);
                    end
                    clear tmp_current tmp_last sum_current sum_last
                        
                end
            else
                best_indices_count = best_indices_count + 1;
                best_ctd_indices(best_indices_count) = sorted_indices(o_index);
            end
            
            last_profile_number = current_profile_number;
        end
    end

end

