function [ nemo_data ] = create_best_guess( p_nemo_data )
%CREATE_BEST_GUESS Estimates and writes the best guess datetime to the given structure.
%   The Iridium time is only being taken if no GPS time is available, and it is
%   not the first profile after a series with xmit_upcast_status="ICE DETECTION".
%
%   Priority that is being used:
%   1) GPS time
%   2) Iridium time
%   3) Internal float clock time
%   
%   
%   Returns: p_nemo_data with an additional struct containing the best
%           guess datetime.
%
%   Parameters:
%   p_nemo_data:      Struct, containing all floats that should be edited.

%% Initialize return variables
nemo_data = false;

%% Parameter check
parameter_error = false; % indicates if a parameter error was raised

if (ischar(p_nemo_data)) % read profiles using read_profiles.m
    if (~isdir(p_nemo_data)) % parameter is not a directory
        parameter_error = true;
    else % read profiles
        nemo_data = nemo.load.data(p_nemo_data);
    end
else if (~iscell(p_nemo_data)) % raise error
        parameter_error = true;
    else
        nemo_data = p_nemo_data; % store nemo data in return variable
    end
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please provide either a ' ...
        'directory path or a struct created by nemo.load.data!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Estimate best datetime

% run through every float/profile
nemo_data = p_nemo_data;
float_count = length(p_nemo_data);
for o_float = 1:float_count

    current_float = p_nemo_data{o_float};
    profile_count = length(current_float);

    for o_profile_index = 1:profile_count
        % get internal clock (software)
        software_datevec = ...
            datevec( ...
            datetime([ ...
            fliplr(current_float{o_profile_index}.OVERALL_MISSION_INFORMATION.start_date), ...
            current_float{o_profile_index}.OVERALL_MISSION_INFORMATION.start_time 0 ...
            ]) ...
            + seconds(current_float{o_profile_index}.PROFILE_TECHNICAL_DATA.xmit_surface_start_time) ...
            );
        software_time = software_datevec(4:6);
        % get internal clock (RTC)
        rtc_time = current_float{o_profile_index}.PROFILE_DATA( ...
            current_float{o_profile_index}.PROFILE_DATA(:, 7) == min(current_float{o_profile_index}.PROFILE_DATA(:, 7)), ...
            2:4);
        % if there are multiple pressure levels get the one with the latest
        % time
        rtc_time = sortrows(rtc_time, [1:3]);
        
        if isempty(rtc_time) || isempty(software_datevec) % prevent errors if empty
           current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_datevec = NaN(1, 6);
           current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_datevec_src = 'none';
           continue;
        end
        
        rtc_time = rtc_time(1, :);
        
        current_datevec = datevec(datetime([software_datevec(1:3), rtc_time]));
        
        if (any(rtc_time(1) == 21:23) && any(software_time(1) == 0:3))
            % RTC is more trustworthy, so reduce the software date by 1
            current_datevec = datevec(datetime([software_datevec(1:3), rtc_time]) - days(1));
        elseif (any(rtc_time(1) == 0:3) && any(software_time(1) == 21:23))
            % same here, but now add 1 day
            current_datevec = datevec(datetime([software_datevec(1:3), rtc_time]) + days(1));
        end
        
        current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_datevec = current_datevec;
        current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_datevec_src = 'internal_rtc';
%         current_datevec = [];
%         if o_profile_index ~= 1
%             previous_profile = current_float{o_profile_index - 1};
%         else
%             previous_profile = {};
%         end
%         
%         if o_profile_index < profile_count
%             next_profile = current_float{o_profile_index + 1};
%         else
%             next_profile = {};
%         end
%         
%         % decide which time should be taken
%         
%         if nemo.profile.has_gps(current_float{o_profile_index}) ...
%                 && nemo.profile.has_status(current_float{o_profile_index}, 'DONE')
%             % get gps datetime
%             current_datevec = nemo.profile.get_gps(current_float{o_profile_index});
%             current_datevec = current_datevec(8:13);
%             current_source = 'gps';
%         elseif nemo.profile.has_iridium(current_float{o_profile_index}) ...
%                 && nemo.profile.has_status(current_float{o_profile_index}, 'DONE') ...
%                 && nemo.profile.has_status(previous_profile, 'DONE') ...
%                 && (isempty(next_profile) || nemo.profile.verify_iridium(current_float{o_profile_index}, next_profile))
%             % get iridium datetime
%             current_datevec = nemo.profile.get_iridium(current_float{o_profile_index});
%             current_datevec = current_datevec(4:9);
%             current_source = 'iridium';
%         else
%             % get internal clock
%             current_datevec = ...
%                 datevec( ...
%                     datetime([ ...
%                         %fliplr(current_float{o_profile_index}.DEPLOYMENT_INFO.deployment_date), ...
%                         %current_float{o_profile_index}.DEPLOYMENT_INFO.deployment_time 0 ...
%                         fliplr(current_float{o_profile_index}.OVERALL_MISSION_INFORMATION.start_date), ...
%                         current_float{o_profile_index}.OVERALL_MISSION_INFORMATION.start_time 0 ...
%                     ]) ...
%                     + seconds(current_float{o_profile_index}.PROFILE_TECHNICAL_DATA.xmit_surface_start_time) ...
%                 );
%             current_source = 'internal';
%         end
%         % store datetime
%         if (~isfield(current_float{o_profile_index}, 'SECOND_ORDER_INFORMATION'))
%             current_float{o_profile_index}.SECOND_ORDER_INFORMATION = {};
%         end
%         current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_datevec = current_datevec;
%         current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_datevec_src = current_source;
    end
    nemo_data{o_float} = current_float;
end

end

